/**
 * @author kirigaya
 * @datetime
 * @description 实现带有宽度的线的 corner case 部分的代码，具体的原理请看
 * ./design/webgl.drawio 中的 【corner-case 讨论和推导】
 */



function corner() {

}

/**
 * @description cb 是 counter backwards 倒数 的缩写
 * @param {number} x1 
 * @param {number} y1 
 * @param {number} x2 
 * @param {number} y2 
 * @returns {number}
 */
function getSlopeCb(x1, y1, x2, y2) {
    if (x1 === x2) {
        return 0;
    }
    return (x1 - x2) / (y1 - y2);
}


/**
 * @description 计算设计图中的 alpha 和 beta，返回的偏移量一定非负
 * @param {number} x1 
 * @param {number} y1 
 * @param {number} x2 
 * @param {number} y2 
 * @param {number} w 当前 VAO 渲染的宽度
 */
function calcAlpha(x1, y1, x2, y2, w) {
    if (x1 === x2) {
        return 2 * w;
    }

    if (y1 === y2) {
        return 0;
    } else {
        const b = (x1 - x2) / (y1 - y2);
        let alpha = 2 * w;
        // TODO: 检查数值溢出
        if (y1 > y2) {
            alpha = alpha * (Math.sqrt(1 + b * b) + b);
        } else {
            alpha = alpha * (Math.sqrt(1 + b * b) - b);
        }
        return alpha;
    }
}


/**
 * 
 * @param {number[]} p1 
 * @param {number[]} p2 
 * @param {number[]} p3 
 * @param {number} w
 */
export function getCornerTriangle(p1, p2, p3, w) {
    const controlPoints = [];
    const [x2, y2] = p2;

    if (p1 === undefined) {
        controlPoints.push({ x: x2, y: y2, ws: 0 });
        controlPoints.push({ x: x2, y: y2, ws: 4 });
        return controlPoints;
    }

    if (p3 === undefined) {
        controlPoints.push({ x: x2, y: y2, ws: 0 });
        controlPoints.push({ x: x2, y: y2, ws: 4 });
        return controlPoints;
    }

    const [x1, y1] = p1;
    const [x3, y3] = p3;
    
    if (y1 < y2) {
        if (y2 > y3) {
            // case 2
            const c2ws = calcAlpha(x1, y1, x2, y2, w);
            const c3ws = calcAlpha(x2, y2, x3, y3, w);

            const c1 = { x: x2, y: y2, ws: 4 };
            const c2 = { x: x2 - c2ws, y: y2, ws: 0 };
            const c3 = { x: x2 - c3ws, y: y2, ws: 0 };
            controlPoints.push(c1, c2, c3);

        } else if (y2 === y3) {
            // case 6
            const c2ws = calcAlpha(x1, y1, x2, y2, w);

            const c1 = { x: x2, y: y2, ws: 4 };
            const c2 = { x: x2 - c2ws, y: y2, ws: 0 };
            const c3 = { x: x2, y: y2, ws: 0 };
            controlPoints.push(c1, c2, c3);

        } else {
            // case 9
            const c2ws = calcAlpha(x1, y1, x2, y2, w);
            const c3ws = calcAlpha(x2, y2, x3, y3, w);

            const c1 = { x: x2, y: y2, ws: 0 };
            const c2 = { x: x2 - c2ws, y: y2, ws: 4 };
            const c3 = { x: x2 + c3ws, y: y2, ws: 4 };

            controlPoints.push(c1, c2, c3);
        }
    } else if (y1 === y2) {
        if (y2 > y3) {
            // case 5
            const c3ws = calcAlpha(x2, y2, x3, y3, w);

            const c1 = { x: x2, y: y2, ws: 4 };
            const c2 = { x: x2, y: y2, ws: 0 };
            const c3 = { x: x2 + c3ws, y: y2, ws: 0 };
            controlPoints.push(c1, c2, c3);

        } else if (y2 === y3) {
            // case 3
            controlPoints.push({ x: x2, y: y2, ws: 0 });
            controlPoints.push({ x: x2, y: y2, ws: 4 });
        } else {
            // case 4
            const c3ws = calcAlpha(x2, y2, x3, y3, w);
            
            const c1 = { x: x2, y: y2, ws: 0 };
            const c2 = { x: x2, y: y2, ws: 4 };
            const c3 = { x: x2 + c3ws, y: y2, ws: 4 };
            controlPoints.push(c1, c2, c3);
        }
    } else {
        if (y2 > y3) {
            // case 8
            const c2ws = calcAlpha(x1, y1, x2, y2, w);
            const c3ws = calcAlpha(x2, y2, x3, y3, w);

            const c1 = { x: x2, y: y2, ws: 0 };
            const c2 = { x: x2 - c2ws, y: y2, ws: 4 };
            const c3 = { x: x2 + c3ws, y: y2, ws: 4 };
            controlPoints.push(c1, c2, c3);

        } else if (y2 === y3) {
            // case 7
            const c2ws = calcAlpha(x1, y1, x2, y2, w);

            const c1 = { x: x2, y: y2, ws: 0 };
            const c2 = { x: x2 - c2ws, y: y2, ws: 4 };
            const c3 = { x: x2, y: y2, ws: 4 };
            controlPoints.push(c1, c2, c3);

        } else {
            // case 1: y2 < y1 && y2 < y3
            const c2ws = calcAlpha(x1, y1, x2, y2, w);
            const c3ws = calcAlpha(x2, y2, x3, y3, w);

            const c1 = { x: x2, y: y2, ws: 0 };
            const c2 = { x: x2 - c2ws, y: y2, ws: 4 };
            const c3 = { x: x2 + c3ws, y: y2, ws: 4 };
            controlPoints.push(c1, c2, c3);
        }
    }

    return controlPoints;
}